MIDI file generation from results of training on PTGGs or Bach Chorales
Donya Quick

Last modified: 18-June-2014


> module LearningGen where
> import Euterpea hiding (C, D, P)
> import JazzFG
> import ClassicalFG
> import GrammarMonad 
> import GrammarRules
> import System.Random
> import PostProc
> import Learning
> import PCFGtoPTGG
> import Rohrmeier hiding (S)
> import ChordSpaces hiding (i)
> import Data.List
> import TemporalGen
> import RuleUtils
> import Search
> import Constraints


> cInstrs = [Bassoon, EnglishHorn, Clarinet, Oboe] 

> average xs = sum xs / fromIntegral (length xs)

> pcfgGenExamples :: FilePath -> FilePath -> FilePath -> [Int] -> IO ()
> pcfgGenExamples pcfgFile pcfgPFile outFolder1 seeds = do 
>     (startSym, rules) <- readPCFG pcfgFile
>     probs <- readProbsFinal pcfgPFile
>     let avgProbs = map average probs
>         prules = zip avgProbs rules
>         ptgg1 = toPTGG1 qn prules
>         ptgg2 = toPTGG2 qn prules
>         ptgg3 = toPTGG3 qn prules
>         f1 g = gen' ptgg1 8 g Major (NT $ Chord qn TR) -- constant durs
>         f2 g = gen' ptgg2 8 g Major (NT $ Chord 4 TR) -- split durs
>         f3 g = gen' ptgg3 8 g Major (NT $ Chord 4 I) -- split durs
>     lens1 <- sequence $ map (ptggStep outFolder1 1 f1 forceCT) seeds
>     lens2 <- sequence $ map (ptggStep outFolder1 2 f2 forceCT) seeds
>     lens3 <- sequence $ map (ptggStep outFolder1 3 f3 id) seeds
>     writeLensFile outFolder1 lens1 lens2 lens3 seeds
>     putStrLn "PCFG generation complete."

> pcfgGenExamplesE :: FilePath -> FilePath -> FilePath -> [Int] -> IO ()
> pcfgGenExamplesE pcfgFileE pcfgPFileE outFolder1E seeds = do 
>     (startSym, rules) <- readPCFG pcfgFileE
>     probs <- readProbsFinal pcfgPFileE
>     let avgProbs = map average probs
>         prules = zip avgProbs rules
>         ptgg3 = toPTGG3 qn prules
>         f g = gen' ptgg3 8 g Major (NT $ Chord 4 I) -- split durs
>     lens <- sequence $ map (ptggStepTSD outFolder1E 3 f id) seeds
>     writeLensFile2 outFolder1E lens seeds
>     putStrLn "PCFG generation complete."

> showKey :: Key -> String
> showKey (k,m) = (keys !! k) ++ " "++show m where
>     keys = ["C", "C-sharp", "D", "E-flat", "E", "F", 
>             "F-sharp", "G", "A-flat", "A", "B-flat", "B"]

> writeLensFile :: FilePath -> [(Int, Key)] -> [(Int, Key)] -> [(Int, Key)] -> [Int] -> IO ()
> writeLensFile fOut lens1 lens2 lens3 seeds = 
>     let ks = map snd lens1
>         ds = Data.List.transpose [map fst lens1, map fst lens2, map fst lens3]
>         f s d k = "\\bigstrut & "++show s ++ "\t & " ++ 
>                   concatMap (\x -> show x ++ "\t & ") d ++ 
>                   showKey k ++ " \\\\ \n"
>         header = "\\bigstrut & Seed\t"++ concatMap g [1,2,3] ++" & Key \\\\ \n"
>         g i = "& Approach " ++ show i ++ "\t"
>     in  writeFile (fOut++"_lengths.txt") (header ++ concat (zipWith3 f seeds ds ks)) 

> ptggStep :: (Eq a, Show a) => FilePath ->
>     Int -> (StdGen -> Term a) -> (a -> CType) -> Int -> IO (Int, Key)
> ptggStep fp num f ft s = do
>     let [gGen, gKey, gCS, gFG] = take 4 $ splitN $ mkStdGen s
>         t = f gGen
>         t' = tMap ft t -- force to Roman numerals
>         k = fst $ randomR (0,11 :: AbsPitch) gKey -- pick key, but only allow major for now
>         m = detMode k -- determine mode from key
>         aChords = atTrans (k`mod`12) $ toAbsChords t' m
>         justAs = map (\(a,b,c) -> c) aChords
>         csChords = greedyProg satbOP opcEq (noCPL 7) nearFall gCS justAs -- through chord space
>         aChords' = zipWith (\(a,b,c) d -> (a,b,d)) aChords csChords
>         csM = vsToMusicI cInstrs $ toVoices aChords' -- as music
>         fg = snd $ addFG defConsts gFG $ reverse $ toVoices aChords' -- add foreground
>         fgM = vsToMusicI cInstrs $ reverse fg
>     putStr ("Working on grammar "++show num++", seed "++show s++"...") 
>     writeRT fp s "_term" num t t' -- write terms and flattened version
>     writeMT fp s "_cs" num csM -- write chords only
>     writeMT fp s "_fg" num fgM -- write foreground
>     putStrLn "Done."
>     return (length aChords, (k `mod` 12, m))


> ptggStepTSD :: (Eq a, Show a) => FilePath ->
>     Int -> (StdGen -> Term a) -> (a -> CType) -> Int -> IO (Int, Key)
> ptggStepTSD fp num f ft s = do
>     let [gGen, gKey, gCS, gFG, gTSD] = take 5 $ splitN $ mkStdGen s
>         t = f gGen
>         t' = tMap ft t -- force to Roman numerals
>         k = fst $ randomR (0,11 :: AbsPitch) gKey -- pick key, but only allow major for now
>         m = detMode k -- determine mode from key
>         cChords = ctTrans (k`mod`12) $ toChords t' m -- find Roman numerals
>         justCs = map (\(a,b,c) -> c) cChords -- get Roman numeral mapping
>         justCs' =  expandTSD (const True) gTSD justCs -- greedy TSD expansion 
>         aChords = map toAbsChord $ zipWith (\(a,b,c) d -> (a,b,d)) 
>                   cChords justCs -- initial vector mapping
>         aChords' = map toAbsChord $ zipWith (\(a,b,c) d -> (a,b,d)) 
>                    cChords justCs' -- initial vector mapping
>         justAs = map (\(a,b,c) -> c) aChords -- just chords as vectors 
>         justAs' = map (\(a,b,c) -> c) aChords' -- just chords as vectors
>         csChords = greedyProg satbOP opcEq (noCPL 7) nearFall gCS justAs -- through chord space
>         csChords' = greedyProg satbOP opcEq (noCPL 7) nearFall gCS justAs' -- through chord space
>         aChords2 = zipWith (\(a,b,c) d -> (a,b,d)) aChords csChords
>         aChords2' = zipWith (\(a,b,c) d -> (a,b,d)) aChords csChords'
>         csM = vsToMusicI cInstrs $ toVoices aChords2 -- as music
>         csM' = vsToMusicI cInstrs $ toVoices aChords2' -- as music
>         fg = snd $ addFG defConsts gFG $ reverse $ toVoices aChords2 -- add foreground
>         fg' = snd $ addFG defConsts gFG $ reverse $ toVoices aChords2' -- add foreground
>         fgM = vsToMusicI cInstrs $ reverse fg
>         fgM' = vsToMusicI cInstrs $ reverse fg'
>     putStr ("Working on grammar "++show num++", seed "++show s++"...") 
>     writeFile (fp++show s++"_term3e.txt") (show justCs')
>     writeRT fp s "_term" num t t' -- write terms and flattened version
>     writeMT fp s "_cs" num csM -- write chords only
>     writeMT fp s "_fg" num fgM -- write foreground       
>     writeMT fp s "_csx" num csM' -- write chords only
>     writeMT fp s "_fgx" num fgM' -- write foreground      
>     putStrLn "Done."
>     return (length aChords, (k `mod` 12, m))


> tsdSpaceSimple = [[I,III,VI], [IV,II], [V,VII]] :: QSpace CType

> --         I      II    III   IV    V     VI    VII
> rCounts = [12758, 4443, 1451, 3689, 8449, 2061, 1209] :: [Int]

> tsdSpace :: QSpace CType
> tsdSpace = [f I 0 ++ f III 2 ++ f VI 5,
>             f IV 3 ++ f II 1,
>             f V 4 ++ f VII 6] where
>     f x i = take (rCounts !! i) $ repeat x

> expandTSD :: Predicate (CType, CType) -> StdGen -> [CType] -> [CType]
> expandTSD p g xs = greedyProg tsdSpace tsdEq p tsdFall g xs where
>     tsdEq :: EqRel CType
>     tsdEq a b = or $ map (\e -> elem a e && elem b e) tsdSpace
>     tsdFall :: Fallback CType
>     tsdFall es g x = (g,x)

> detMode :: Int -> Mode 
> detMode k = if k < 12 then Major else Minor

> writeMT fOut s fn n m = writeMidi (fOut++show s++fn++show n++".mid") m

> writeRT fOut s fn n t t' = writeFile (fOut++show s++fn++show n++".txt") 
>           (show (flatten t) ++ "\n\n" ++ show (flatten t'))


==================

PTGG

> ptggGenExamples :: FilePath -> FilePath -> FilePath -> [Int] -> Int -> IO ()
> ptggGenExamples ptggFile ptggPFile outFolder2 seeds2 iters = do 
>     (startSym, rules) <- readPTGG ptggFile 
>     probs <- readProbsFinal ptggPFile 
>     let avgProbs = map average probs
>         ptgg = map (toRelDur qn) $ zipWith updateProb avgProbs rules -- construct PTGG
>         f1 g = gen' ptgg iters g Major (i 4) -- standard generation
>     lens1 <- sequence $ map (ptggStep outFolder2 1 f1 id) seeds2
>     writeLensFile2 outFolder2 lens1 seeds2
>     putStrLn "PTGG generation complete."

> updateProb :: Double -> Rule a -> Rule a
> updateProb pNew ((x,pOld) :-> rf) = ((x,pNew) :-> rf)

> writeLensFile2 fOut lens1 seeds = 
>     let ks = map snd lens1
>         ds = Data.List.transpose [map fst lens1]
>         f s d k = "\\bigstrut & "++show s ++ "\t & " ++ 
>                   concatMap (\x -> show x ++ "\t & ") d ++ 
>                   showKey k ++ " \\\\ \n"
>         header = "\\bigstrut & Seed\t"++ concatMap g [1,2,3] ++" & Key \\\\ \n"
>         g i = "& Length " ++ show i ++ "\t"
>     in  writeFile (fOut++"_lengths.txt") (header ++ concat (zipWith3 f seeds ds ks)) 
