Special playback functions
Donya Quick
Last modified: 26-Sept-2014

Kulitta-specific playback functions.

> module PlayK where
> import Euterpea
> import Codec.Midi
> import Sound.PortMidi
> import System.IO.Unsafe (unsafePerformIO)
> import Euterpea.IO.MIDI.MidiIO
> import System.Directory
> -- import Play2 
> import Euterpea.ExperimentalPlay


Play a MIDI file to a user-specified device. Unlike the regular play function,
this version also checks whether the file exists to avoid crashing the calling
program. If the file doesn't exist, an error message will be printed.

> play' fp devID = do
>   x <- tryImportFile fp 
>   seq x $ case x of Left err -> putStrLn ("Error: "++err) >> return ()
>                     Right m -> playM' devID m

> playX fp devID chanOffset = do
>   x <- tryImportFile fp 
>   seq x $ case x of Left err -> putStrLn ("Error: "++err) >> return ()
>                     Right m -> playM' devID (trackMod chanOffset m)

> writeX fp m chanOffset = 
>   let x = trackMod chanOffset $ testMidi m 
>   in  exportMidiFile fp x

writeMidi fn = exportMidiFile fn . testMidi

> trackMod x m = 
>     let t = tracks m
>         t' = map (map (trackMod' x)) t 
>     in  m{tracks = t'} 

> trackMod' :: Channel -> (Ticks, Message) -> (Ticks, Message)
> trackMod' x (a, NoteOff c k v) = (a, NoteOff (c+x) k v)
> trackMod' x (a, NoteOn c k v) = (a, NoteOn (c+x) k v)
> trackMod' x (a, ProgramChange c p) = (a, ProgramChange (c+x) p)
> trackMod' x (a, ControlChange c v1 v2) = (a, ControlChange (c+x) v1 v2)
> trackMod' x (a,v) = (a,v)

> playF fp fmid devID = do
>   x <- tryImportFile fp 
>   seq x $ case x of Left err -> putStrLn ("Error: "++err) >> return ()
>                     Right m -> playM' devID (fmid m)

Code to check whether the file exists:

> tryImportFile fp = do
>     let theDir = getDir fp
>         theFile = getFile fp
>     files <- getDirectoryContents theDir
>     if elem theFile files then importFile fp >>= return 
>         else return $ Left ("File "++fp ++ " does not exist, so it cannot be played!") where
>         getDir fp = reverse $ dropWhile (not.(`elem` "/\\")) $ reverse fp
>         getFile fp = drop (length $ getDir fp) fp

=====================

DATA MANIPULATION FUNCTIONS

For playback, it's useful to be able to set the volume on some systems.

> setVol :: Int -> Music Pitch -> Music1
> setVol v = mMap (\p -> (p, [Volume v])) 

> setVol1 :: Int -> Music1 -> Music1
> setVol1 v = mMap (\(p,nas) -> (p, Volume v : filter f nas)) where
>     f (Volume v) = False
>     f _ = True

Sometimes it is also useful to play to devices with instrument
information stripped (i.e. sending to a single synth such that 
only one channel can be used).

> stripInstrs :: Music a -> Music a
> stripInstrs (a :+: b) = stripInstrs a :+: stripInstrs b
> stripInstrs (a :=: b) = stripInstrs a :=: stripInstrs b
> stripInstrs (Prim p) = Prim p
> stripInstrs (Modify (Instrument i) m) = stripInstrs m
> stripInstrs (Modify c m) = Modify c $ stripInstrs m

> m2m :: (Music1 -> Music1) -> Midi -> Midi
> m2m fm1 midi = 
>     let (m1, _, _) = fromMidi midi
>     in   testMidi $ fm1 m1

> ch1 :: Int -> Midi -> Midi
> ch1 v = if v <=0 then m2m stripInstrs
>         else m2m (setVol1 v . stripInstrs)