PCFG to PTGG Conversion
Donya Quick

Last modified: 21-Mar-2014

Module for turning a PCFG into a PTGG for use with Kulitta's 
generative algorithms.

> module PCFGtoPTGG where
> import Euterpea hiding (C, D, P)
> import Rohrmeier 
> import Parser
> import GrammarMonad
> import RuleUtils


> toTerm d x = NT $ Chord d x

> s' = Rohrmeier.S :: RTerm

Make a PTGG using constant durations:

> ptggRule1 :: Dur -> (Double, Parser.Rule a ) -> GrammarMonad.Rule a
> ptggRule1 dConst (p, (lhs,rhs)) = 
>     (lhs,p) :-> \c d -> GrammarMonad.S$ map (toTerm dConst) rhs where

> toPTGG1 :: Dur -> [(Double, Parser.Rule a )] -> [GrammarMonad.Rule a]
> toPTGG1 d = map (ptggRule1 d)


Make a PTGG using temporal divisions of 2 and 4 and a minimum duration:

> ptggRule2 :: (Double, Parser.Rule a ) -> GrammarMonad.Rule a
> ptggRule2 (p, (lhs, rhs)) = (lhs,p) :-> \c d -> durPats d rhs where
>     durPats d xs = case xs of 
>         [x] -> NT $ Chord d x
>         [x1,x2] -> GrammarMonad.S $ zipWith toTerm [d/2, d/2] [x1,x2]
>         [x1,x2,x3] -> 
>             GrammarMonad.S $ zipWith toTerm [d/4, d/4, d/2] [x1,x2,x3] 
>         [x1,x2,x3,x4] -> 
>             GrammarMonad.S $ zipWith toTerm (repeat (d/4)) [x1,x2,x3,x4]
>         _ -> error ("(toPTGG2) Bad rule rank: "++show (length xs))

> toPTGG2 :: Dur -> [(Double, Parser.Rule a )] -> [GrammarMonad.Rule a]
> toPTGG2 d = map (toRelDur d . ptggRule2)


Make a PTGG Using only CTypes

> ptggRule3 :: (Double, Parser.Rule RTerm ) -> GrammarMonad.Rule CType
> ptggRule3 (d, (lhs, rhs)) = ptggRule2 (d, (forceCT lhs, map forceCT rhs))

> toPTGG3 :: Dur -> [(Double, Parser.Rule RTerm )] -> [GrammarMonad.Rule CType]
> toPTGG3 d = normalize . map (toRelDur d . ptggRule3)




Utility for forcing conversion of generated Terms. TR and T are forced to
I, DR and D are forced to V, and so on. P in this case is "plagal" (which 
resolves to I), but this interpretation would also work for P as "phrase."

> forceCT :: RTerm -> CType
> forceCT (C ct) = ct
> forceCT r = 
>     let rts = [Piece, TR, DR, SR, T, D, s', T, TP, TCP, SP, DP, P] :: [RTerm]
>         cts = [I,     I,  V,  IV, I, V, IV, I, I,  I,   IV, V,  I] :: [CType] 
>         rcts = zip rts cts :: [(RTerm, CType)]
>     in  case lookup r rcts of Just y -> y
>                               Nothing -> error "(forceCT) Unhandled constructor" 
